from typing import Union, BinaryIO, List
from ._util import DecoderType
from mutf8 import decode_modified_utf8
from ._named_tag import NamedTag

class ReadContext:
    offset: int

def load(
    filepath_or_buffer: Union[str, bytes, BinaryIO, memoryview, None],
    *,
    compressed: bool = True,
    little_endian: bool = False,
    read_context: ReadContext = None,
    string_decoder: DecoderType = decode_modified_utf8,
) -> NamedTag:
    """Load one binary NBT object.

    :param filepath_or_buffer: A string path to a file on disk, a bytes or memory view object containing the binary NBT or a file-like object to read the binary data from.
    :param compressed: Is the binary data gzip compressed.
    :param little_endian: Are the numerical values stored as little endian. True for Bedrock, False for Java.
    :param read_context: Optional ReadContext object to get read end offset
    :param string_decoder: The bytes decoder function to parse strings. mutf8.decode_modified_utf8 for Java, amulet_nbt.utf8_escape_decoder for Bedrock.
    :raises: NBTLoadError if an error occurred when loading the data.
    """
    ...

def load_array(
    filepath_or_buffer: Union[str, bytes, BinaryIO, memoryview, None],
    *,
    count: int = 1,
    compressed: bool = True,
    little_endian: bool = False,
    read_context: ReadContext = None,
    string_decoder: DecoderType = decode_modified_utf8,
) -> List[NamedTag]:
    """Load an array of binary NBT objects from a contiguous buffer.

    :param filepath_or_buffer: A string path to a file on disk, a bytes or memory view object containing the binary NBT or a file-like object to read the binary data from.
    :param count: The number of binary NBT objects to read. Use -1 to exhaust the buffer.
    :param compressed: Is the binary data gzip compressed. This only supports the whole buffer compressed as one.
    :param little_endian: Are the numerical values stored as little endian. True for Bedrock, False for Java.
    :param read_context: Optional ReadContext object to get read end offset
    :param string_decoder: The bytes decoder function to parse strings. mutf8.decode_modified_utf8 for Java, amulet_nbt.utf8_escape_decoder for Bedrock.
    :raises: NBTLoadError if an error occurred when loading the data.
    """
    ...



import unittest
from copy import copy

import amulet_nbt
from binary_data import binary_data_tuple


class NBTTests(unittest.TestCase):
    def _load(self, b: bytes, little_endian: bool = False):
        b_copy = copy(b)
        named_tag = amulet_nbt.load(b_copy, little_endian=little_endian)
        self.assertEqual(b, b_copy, msg="The buffer changed.")
        named_tag2 = amulet_nbt.load(b_copy, little_endian=little_endian)
        self.assertEqual(named_tag.name, named_tag2.name)
        self.assertEqual(named_tag.tag, named_tag2.tag)
        return named_tag

    def test_read_big_endian(self):
        for data in binary_data_tuple:
            self.assertEqual(
                data.named_tag, self._load(data.big_endian), msg=str(data.named_tag)
            )

    def test_read_big_endian_compressed(self):
        for data in binary_data_tuple:
            self.assertEqual(
                data.named_tag,
                amulet_nbt.load(data.big_endian_compressed),
                msg=str(data.named_tag),
            )

    def test_read_little_endian(self):
        for data in binary_data_tuple:
            self.assertEqual(
                data.named_tag,
                self._load(data.little_endian, little_endian=True),
                msg=str(data.named_tag),
            )

    def test_read_little_endian_compressed(self):
        for data in binary_data_tuple:
            self.assertEqual(
                data.named_tag,
                amulet_nbt.load(data.little_endian_compressed, little_endian=True),
                msg=str(data.named_tag),
            )

    def test_write_big_endian(self):
        for data in binary_data_tuple:
            self.assertEqual(
                data.named_tag.to_nbt(compressed=False),
                data.big_endian,
                msg=str(data.named_tag),
            )

    def test_write_little_endian(self):
        for data in binary_data_tuple:
            self.assertEqual(
                data.named_tag.to_nbt(compressed=False, little_endian=True),
                data.little_endian,
                msg=str(data.named_tag),
            )


if __name__ == "__main__":
    unittest.main()



import unittest
import os
import sys
from template import TempitaManager

ROOT_PATH = os.path.dirname(os.path.dirname(__file__))


class TestString(unittest.TestCase):
    @unittest.skipUnless(
        sys.version_info[:2] == (3, 9),
        "Different python versions produce different outputs.",
    )
    def test_is_compiled(self):
        """Check if the files have been baked out."""
        for file in TempitaManager().files:
            if file.changed:
                raise self.failureException(
                    f"Tempita file {file.rel_path} has not been compiled."
                )


if __name__ == "__main__":
    unittest.main()
